/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.apache.harmony.jpda.tests.jdwp.ThreadReference;

import org.apache.harmony.jpda.tests.framework.DebuggeeSynchronizer;
import org.apache.harmony.jpda.tests.framework.LogWriter;
import org.apache.harmony.jpda.tests.share.JPDADebuggeeSynchronizer;
import org.apache.harmony.jpda.tests.share.SyncDebuggee;


/**
 * The class specifies debuggee for <code>org.apache.harmony.jpda.tests.jdwp.ThreadReference.OwnedMonitorsStackDepthInfoTest</code>.
 * This debuggee starts the tested thread <code>TESTED_THREAD</code> which
 * synchronized with test via the <code>SGNL_READY</code> and
 * <code>SGNL_CONTINUE</code> signals.
 */
public class OwnedMonitorsStackDepthInfoDebuggee extends SyncDebuggee {

    public static final String TESTED_THREAD = "TestedThread";

    // These two objects are used
    static Object waitForStart = new Object();
    static Object waitForFinish = new Object();

    public void run() {
        DebuggeeThread thrd = new DebuggeeThread(TESTED_THREAD,
                logWriter, synchronizer);

        synchronized(waitForStart){
            thrd.start();
            try {
                waitForStart.wait();
            } catch (InterruptedException e) {

            }
        }

        synchronized(waitForFinish){
            logWriter.println("thread is finished");
        }
    }

    class DebuggeeThread extends Thread {

        LogWriter logWriter;
        DebuggeeSynchronizer synchronizer;

        public DebuggeeThread(String name, LogWriter logWriter,
                DebuggeeSynchronizer synchronizer) {
            super(name);
            this.logWriter = logWriter;
            this.synchronizer = synchronizer;
        }

        // Deliberately make several stack frames with known monitor states.

        // 1. No monitors held in the outermost frame.
        public void run() {
            run1();
        }

        // 2. One monitor ("this") held in the next frame.
        private synchronized void run1() {
            run2();
        }

        // 3. No monitors held in the next frame.
        private void run2() {
            run3();
        }

        // 4. Two monitors held in the frame after that.
        private void run3() {
            synchronized(OwnedMonitorsStackDepthInfoDebuggee.waitForFinish){

                synchronized(OwnedMonitorsStackDepthInfoDebuggee.waitForStart){

                    OwnedMonitorsStackDepthInfoDebuggee.waitForStart.notifyAll();

                    logWriter.println(getName() +  ": started");
                    synchronizer.sendMessage(JPDADebuggeeSynchronizer.SGNL_READY);

                    logWriter.println(getName() +  ": wait for SGNL_CONTINUE");
                    synchronizer.receiveMessage(JPDADebuggeeSynchronizer.SGNL_CONTINUE);
                    logWriter.println(getName() +  ": finished");
                }
            }
        }
    }

    public static void main(String [] args) {
        runDebuggee(OwnedMonitorsStackDepthInfoDebuggee.class);
    }
}
